/*
    AddEvent.rexx Macro
    Adds events to calendars created by FWCalendar.rexx
    $VER: FWCAddEvent.rexx v3.64 (20 Sep 1999)
    Ron Goertz (goertz@earthlink.net)
*/
 
OPTIONS RESULTS
signal on syntax

PortList   = show('P')
ErrorCount = 0

call AddLibraries
bguiopen = bguiopen()
if ErrorCount > 0 then call Cleanup

parse source . . . FullCallPath . CallHost
CallHost = strip(CallHost)
ScriptDir = PathPart(FullCallPath)

CurrentDir = upper(Pragma('D'))
if right(CurrentDir, 1) ~= ':' then CurrentDir = CurrentDir'/'

if (pos('FINALWRITER', CurrentDir) > 0) | (left(CallHost, 6) == 'FINALW') then do
  App     = 'FW'
  AppName = 'FINALWRITER'
  if CallHost == 'REXX' then address value substr(PortList, pos('FINALW.', PortList), 8)
  GETDOCITEMPREFS Decimal; DecimalFormat = result
  DOCITEMPREFS Decimal Period
end
else if (pos('PAGESTREAM', CurrentDir) > 0) | (CallHost == 'PAGESTREAM') then do
  App     = 'PGS'
  AppName = 'PAGESTREAM'
  address 'PAGESTREAM'
end

call SetVariables
call GetLogInfo

Month = substr(TempDate,5,2)
if left(Month,1) == "0" then Month = right(Month,1)
PrevMonth = Month - 1
if PrevMonth = 0 then PrevMonth = 12
NextMonth = Month + 1
if NextMonth = 13 then NextMonth = 1
 
Year = left(TempDate,4)
if (Year//4 == 0 & Year//100 > 0) | Year//400 == 0 Then MonthLength.2  = 29

interpret "StartDate = Day."Date('W', TempDate, 'S')
if (DoExtended == 0) | (StartDate + MonthLength.Month > 35) then MaxDate = MonthLength.Month
else MaxDate = 35 - StartDate

call GetEvent
exit

/*********************************************/
/*              Subroutines                  */
/*********************************************/
/***//*******  AddLibraries () Subroutine  ***********/
AddLibraries:
  RequiredLibs = 'rexxsupport.library rexxbgui.library'
  do i = 1 to words(RequiredLibs)
    lib = word(RequiredLibs, i)
    if exists('LIBS:'lib) then call addlib(lib, 0, -30, 0)
    else call AddMsg('E', lib' is required but could not be found.')
  end
  if ~exists('LIBS:bgui.library') then call AddMsg('E', 'bgui.library is required but could not be found.')

  if exists('LIBS:rexxtricks.library') then do
    call addlib('rexxtricks.library', 0, -30, 0)
    RexxTricks = 1
  end

  return
/**/

/***//*******  AddMsg (AM) Subroutine  ***********/
AddMsg:
  parse arg AM_MsgType, AM_Msg

  if AM_MsgType == 'E' then do
    ErrorCount = ErrorCount + 1
    Error.ErrorCount = AM_Msg
  end
  else do
    WarningCount = WarningCount + 1
    Warning.WarningCount = AM_Msg
  end

  return
/**/

/***//*******  Cleanup () Subroutine  ***********/
Cleanup:
  signal off syntax

  if VariablesSet == 1 then do
    interpret UserPrefs
    if Req ~= 0 then call bguiwinclose(Req)
    if App == 'FW' then do
      SELECTOBJECT
      REDRAW
      if upper(DecimalFormat) == 'COMMA' then DocItemPrefs Decimal Comma
    end
    else if App == 'PGS' then do
      SELECTOBJECT None WINDOW winName
      if WindowRefreshed ~= 1 then do
        REFRESH ON
        REFRESHWINDOW WINDOW winName
      end
    end
  end

  LogOpen = open('FWCLog', Storage'FWCLog.txt', 'W')
  if LogOpen == 1 then OutType = 'File'
  if (ErrorCount > 0) & (LogOpen == 0) then do
    LogOpen = 1
    call open('FWCLog', 'CON:10/10/500/300/FWCalendar.rexx Message/WAIT/CLOSE')
    OutType = 'CON'
  end

  if LogOpen == 1 then do
    call writeln('FWCLog', '      Macro: 'strip(substr(sourceline(4), pos(':', sourceline(4)) + 1)))
    call writeln('FWCLog', 'Application: 'PgmVersion)
    call writeln('FWCLog', 'Current Dir: 'CurrentDir)
    call writeln('FWCLog', ' Script Dir: 'ScriptDir)
    call writeln('FWCLog', '       Host: 'CallHost)
    call writeln('FWCLog', '   Calendar: 'Month.Month' 'Year||'0a'x)
  end

  if ErrorCount > 0 then do
    do i = 1 to ErrorCount
      call writeln('FWCLog', Error.i)
    end

    if exists(ScriptDir''File2Use) then do
      call writeln('FWCLog', '0a'x||' -- 'ScriptDir''File2Use' -- ')
      call open('DataFile', ScriptDir''File2Use)
        do until eof('DataFile')
          Ln = ReadLn('DataFile')
          if pos('End Pass One', Ln) > 0 then leave
          call writeln('FWCLog', Ln)
        end
      call close('DataFile')
    end

    if EventFile ~= '' then do
      call writeln('FWCLog', '0a'x||' -- 'EventFile' -- ')
      call open('DataFile', EventFile)
        do until eof('DataFile')
          call writeln('FWCLog', ReadLn('DataFile'))
        end
      call close('DataFile')
    end

    FileMsg = 'Error ... see 'Storage'FWCLog.txt for details.'||'0a'x||'Forward log file to: Ron Goertz <goertz@earthlink.net>'||'0a'x||'if you are unable to resolve the problem.'
    Conbgui = 'Error ... see the shell output for details.'||'0a'x||'Forward contents of output to'||'0a'x||'Ron Goertz <goertz@earthlink.net>'||'0a'x||'if you are unable to resolve the problem.'
    ConCon  = 'Error ... see the output above for details.'||'0a'x||'Forward contents of output to'||'0a'x||'Ron Goertz <goertz@earthlink.net>'||'0a'x||'if you are unable to resolve the problem.'
    if (OutType == 'File') & (bguiopen == 1) then call bguireq('1B'x||'c'FileMsg,'*'OK$,'FWCAddEvent notice',,AppScreen)
    if (OutType == 'File') & (bguiopen == 0) then do
      call open('CON', 'CON:10/10/500/300/FWCAddEvent notice/WAIT/CLOSE')
        call writeln('CON', FileMsg)
      call close('CON')
    end
    if (OutType == 'CON') & (bguiopen == 1) then call bguireq('1B'x||'c'Conbgui,'*'OK$,'FWCAddEvent notice',,AppScreen)
    if (OutType == 'CON') & (bguiopen == 0) then call Writeln('FWCLog', '0a'x||ConCon)
  end
  else do
    address command 'delete >NIL: 'Storage'FWC'App'Temp.txt quiet'
    if LogOpen == 1 then call writeln('FWCLog', 'No errors.')
  end

  address command 'delete >NIL: 'Storage'FWCTemp quiet'
  call close('FWCLog')
  if bguiopen = 1 then call bguiclose()
  if DefScreen ~= '' then call setdefaultpubscreen(DefScreen)
  exit
/**/

/***//*******  ConvertDay (CD) Subroutine ***********/
ConvertDay:
  parse arg CD_Day
  If upper(left(CD_Day,1)) == "P" then CD_Day = substr(CD_Day,2) - MonthLength.PrevMonth
  If upper(left(CD_Day,1)) == "N" then CD_Day = substr(CD_Day,2) + MonthLength.Month
  return CD_Day
/**/

/***//*******  DrawBox (DB) Subroutine  ***********/
DrawBox:
  parse arg DB_x1, DB_y1, DB_Width, DB_Height, DB_Weight, DB_Color, DB_FillBool, DB_FillColor, DB_Tint

  if App == 'FW' then do
    if DB_Weight == 'HL' then DB_Weight = 'Hairline'
    else if DB_Weight == 0 then do
      DB_Weight = 'None'
      DB_Color = DB_FillColor
    end

    if DB_FillBool == 1 then DB_FillBool = 'Solid'
    else do
      DB_FillBool = 'Transparent'
      DB_FillColor = DB_Color
    end

    BOXPREFS LINEWT DB_Weight LINECOLOR '"'DB_Color'"' FILL DB_FillBool FILLCOLOR '"'DB_FillColor'"'
    DRAWBOX 1 DB_x1 DB_y1 DB_Width DB_Height; DB_id = result
  end
  else if App == 'PGS' then do
    if DB_Weight == 'HL' then DB_Weight = 0.3pt
    else DB_Weight = DB_Weight'pt'

    if DB_FillBool == 1 then DB_FillBool = 'ON'
    else DB_FillBool = 'OFF'

    If DB_Weight == 0 then DB_LineBool = 'OFF'
    else DB_LineBool = 'ON'

    DRAWBOX DB_x1 DB_y1 DB_x1 + DB_Width DB_y1 + DB_Height WINDOW winName; DB_id = result
    STROKED DB_LineBool OBJECTID DB_id WINDOW winName
    SETSTROKEWEIGHT DB_Weight STROKENUMBER 0 OBJECTID DB_id WINDOW winName
    SETCOLORSTYLE '"'DB_Color'"' COLORNUMBER 0 STROKENUMBER 0 OBJECTID DB_id WINDOW winName
    FILLED DB_FillBool OBJECTID DB_id WINDOW winName
    SETCOLORSTYLE '"'DB_FillColor'"' COLORNUMBER 0 FILL OBJECTID DB_id WINDOW winName
    SETCOLORTINT DB_Tint FILL OBJECTID DB_id WINDOW winName
  end
  return DB_id
/**/

/***//*******  GetEvent (GE) Subroutine  ***********/
GetEvent:
  do GE_i = 0 to 15
    linelist_.GE_i = GE_i
  end
  linelist_.COUNT = min(RowsThatFit, 16)
  call bguilist("eventlist_",Event$,File$)

  GE_StartOrEnd   = 1
  GE_StartDate    = ""
  GE_EndDate      = ""
  GE_Boxed.0      = ""
  GE_Boxed.128    = "B"
  GE_Weekly.0     = ""
  GE_Weekly.128   = "W"
  GadID.          = ''
  GE_Arg.         = ''
  GE_i            = 0
  GE_Day          = 0
  GE_PrevDay      = MonthLength.PrevMonth - StartDate
  GE_NextDay      = 0

  Req = OpenBusy(PrepReq$'...', 45)
  do while (GE_i < 6)
    GE_j = 0
    do while (GE_j < 7)
      call UpdateBusy(Req, 1)
      GE_SerialPosition = (GE_i * 7) + GE_j
      GE_Button = GE_SerialPosition + 1
      if (GE_SerialPosition >= StartDate) & (GE_SerialPosition < StartDate + MonthLength.Month) then Do
        GE_Day = GE_Day + 1
        interpret "GadID."GE_Button" = bguibutton('"GE_Button"_', GE_Day)"
        GadID = GetID(GE_Button'_')
        GE_Arg.GadID = 'C 'left(Month.Month, 3)' 'GE_Day
      end
      else do
        if GE_SerialPosition < StartDate then Do
          GE_PrevDay = GE_PrevDay + 1
          interpret "GadID."GE_Button" = bguibutton('"GE_Button"_', GE_PrevDay)"
          GadID = GetID(GE_Button'_')
          GE_Arg.GadID = 'P 'left(Month.PrevMonth, 3)' 'GE_PrevDay
        end
        else do
          GE_NextDay = GE_NextDay + 1
          interpret "GadID."GE_Button" = bguibutton('"GE_Button"_', GE_NextDay)"
          GadID = GetID(GE_Button'_')
          GE_Arg.GadID = 'N 'left(Month.NextMonth, 3)' 'GE_NextDay
        end
      end
      GE_j = GE_j + 1
    end
    GE_i = GE_i + 1
    if GE_SerialPosition >= StartDate + MonthLength.Month - 1 then leave
  end

  DateButtons = bguihgroup(GadID.1""GadID.2""GadID.3""GadID.4""GadID.5""GadID.6""GadID.7)||,
                bguihgroup(GadID.8""GadID.9""GadID.10""GadID.11""GadID.12""GadID.13""GadID.14)||,
                bguihgroup(GadID.15""GadID.16""GadID.17""GadID.18""GadID.19""GadID.20""GadID.21)||,
                bguihgroup(GadID.22""GadID.23""GadID.24""GadID.25""GadID.26""GadID.27""GadID.28)
  if GE_i > 4 then DateButtons = DateButtons''bguihgroup(GadID.29""GadID.30""GadID.31""GadID.32""GadID.33""GadID.34""GadID.35)
  if GE_i > 5 then DateButtons = DateButtons''bguihgroup(GadID.36""GadID.37""GadID.38""GadID.39""GadID.40""GadID.41""GadID.42)

  g=bguivgroup(,
    bguihgroup(,
      bguicycle("eventtype_",,"eventlist_")bguilayout(LGO_FixMinWidth,1,LGO_FixMinHeight,1)||,
      bguistring("event_",,,256)bguilayout(LGO_FixMinHeight,1),
    )||,
    bguihgroup(,
      bguivgroup(,
        bguiinfo('dummy_',,esc'c'Month.Month)bguilayout(LGO_FixMinHeight, 1)||,
        bguihgroup(,
          bguiinfo("dummy_",,esc"c"left(Day.0,1))||,
          bguiinfo("dummy_",,esc"c"left(Day.1,1))||,
          bguiinfo("dummy_",,esc"c"left(Day.2,1))||,
          bguiinfo("dummy_",,esc"c"left(Day.3,1))||,
          bguiinfo("dummy_",,esc"c"left(Day.4,1))||,
          bguiinfo("dummy_",,esc"c"left(Day.5,1))||,
          bguiinfo("dummy_",,esc"c"left(Day.6,1)),
        )||,
        DateButtons,
      )||,
      bguivgroup(,
        bguiinfo("startchoice_",esc"r"Start$,"")||,
        bguiinfo("endchoice_",esc"r"End$,"")||,
        bguicycle("linechoice_",esc"r"Line$,"linelist_")||,
        bguicheckbox("boxchoice_",esc"r"Boxed$,0)bguilayout(LGO_FixMinWidth,1,LGO_FixMinHeight,1)||,
        bguistring("colorchoice_",esc"r"BoxColor$,,20)||,
        bguicheckbox("weeklychoice_",esc"r"Weekly$,0)bguilayout(LGO_FixMinWidth,1,LGO_FixMinHeight,1)||,
        bguihgroup(,
          bguibutton("OK_",OK$)||,
          bguibutton("cancel_",Cancel$),
        ),
      ),
    ),
  ,"-1","-1")

  call UpdateBusy(Req, 1)
  GE_winID=bguiwindow(EnterEventInfo$,g,5,0,,AppScreen)
  call UpdateBusy(Req, 1)
  call bguiset(obj.linechoice_,GE_winID,CYC_Active,1)
  call bguiset(obj.event_,,BT_Key,EventKey)

  if bguiwinopen(GE_winID)=0 then bguierror(12)

  if Req ~= 0 then call bguiwinclose(Req)
  Req = 0

  id=0
  do while 1
    call bguiwinwaitevent(GE_winID,"ID")
    select
      when (id=id.cancel_) | (id=id.winclose) then call Cleanup
      when id=id.winactive then nop
      when id=id.wininactive then nop
      when id=id.event_ then nop
      when id=id.linechoice_ then nop
      when id=id.boxchoice_ then nop
      when id=id.colorchoice_ then nop
      when id=id.weeklychoice_ then nop
      when id=id.ok_ then do
        GE_EventValue = bguiget(obj.event_, STRINGA_TextVal)
        GE_BoxValue   = bguiget(obj.boxchoice_, GA_Selected)
        if GE_StartDate = "" then call bguireq(EnterStartDate$'...','*'OK$,'FWCAddEvent 'Notice$,,AppScreen)
        else if (GE_EventValue == "") & (GE_Boxed.GE_BoxValue == "") then call bguireq(EnterEvent$'...','*'OK$,'FWCAddEvent 'Notice$,,AppScreen)
        else do
          GE_WeeklyValue  = bguiget(obj.weeklychoice_, GA_Selected)
          GE_EventType = bguiget(obj.eventtype_, CYC_Active)
          GE_BoxColor  = bguiget(obj.colorchoice_, STRINGA_TextVal)

          EventData = Type.GE_EventType", "strip(GE_StartDate)", "strip(GE_EndDate)
          EventData = EventData", "bguiget(obj.linechoice_, CYC_Active)
          EventData = EventData", "GE_Boxed.GE_BoxValue""GE_Weekly.GE_WeeklyValue
          EventData = EventData", "bguiget(obj.colorchoice_, STRINGA_TextVal)
          EventData = EventData", "GE_EventValue

          call bguiwinclose(GE_winID)
          call ProcessEvent
          call bguiwinopen(GE_winID)

          GE_StartOrEnd = 1
          GE_StartDate  = ""
          GE_EndDate    = ""
          call bguiset(obj.startchoice_,GE_winID,INFO_TextFormat,'')
          call bguiset(obj.endchoice_,GE_winID,INFO_TextFormat,'')
        end
      end
      when id=id.eventtype_ then do
        GE_EventType = bguiget(obj.eventtype_, CYC_Active)
        if Type.GE_EventType == Event$ then call bguiset(obj.event_,GE_winID,STRINGA_TextVal,"")
        else do
          GE_DataFile = bguifilereq(ScriptDir''"FWCAddEvent.data", SelectFile$, GE_winID,DOPATTERNS,PatVar)
          if ~exists(GE_DataFile) then do
            call bguireq(GE_DataFile' 'CantFind$'...','*'OK$,'FWCAddEvent 'Notice$,,AppScreen)
            GE_DataFile = ''
          end
          if GE_DataFile == '' then call bguiset(obj.eventtype_, GE_winID, CYC_Active, 0)
          else call bguiset(obj.event_, GE_winID, STRINGA_TextVal,GE_DataFile)
        end
      end
      otherwise do
        GE_StartOrEnd = 1 - GE_StartOrEnd
        GE_ReturnDate = strip(substr(GE_Arg.id, 1, 1)""right(GE_Arg.id, 2), "B", "C")
        Date = substr(GE_Arg.id, 3)
        if GE_StartOrEnd == 0 then do
          call bguiset(obj.startchoice_,GE_winID,INFO_TextFormat,esc"l"Date)
          GE_StartDate = GE_ReturnDate
        end
        else do
          call bguiset(obj.endchoice_,GE_winID,INFO_TextFormat,esc"l"Date)
          GE_EndDate = GE_ReturnDate
        end
      end
    end
  end
  exit
/**/

/***//*******  GetHeight (GH) Subroutine  ***********/
GetHeight:
  parse arg GH_FontType

  if App == 'FW' then do
    TEXTBLOCKTYPEPREFS SIZE FSize.GH_FontType FONT Font.GH_FontType
    DRAWTEXTBLOCK 1 1 1 'A'; GH_id = result
    GETOBJECTCOORDS GH_id; Parse Var result . . . . GH_Text.Height
    DELETEOBJECT GH_id
  end
  else if App == 'PGS' then do
    DRAWTEXTOBJ 0 0 WINDOW winName; GH_id = result
    SELECTTEXT AT 0 0 WINDOW winName
    BEGINCOMMANDCAPTURE
      SETTYPESIZE FSize.GH_FontType WINDOW winName
      SETFONT Font.GH_FontType WINDOW winName
    ENDCOMMANDCAPTURE
    INSERT 'A' WINDOW winName
    GETTEXTOBJ POSITION GH_Text OBJECTID GH_id WINDOW winName
    GH_Text.Height = GH_Text.Bottom - GH_Text.Top
    DELETEOBJECT OBJECTID GH_id WINDOW winName
  end
  return GH_Text.Height
/**/

/***//*******  GetID (GI) Subroutine  ***********/
GetID:
parse arg GI_var

return id.GI_var
/**/

/***//*******  GetLogInfo () Subroutine  ***********/
GetLogInfo:
  if ~exists(Storage'FWC'App'Temp.txt') then address command 'list >'Storage'FWC'App'Temp.txt 'AppName'#? lformat %N'
  if open('Temp', Storage'FWC'App'Temp.txt') ~= 0 then do
    do while ~eof('Temp')
      PgmName = readln('Temp')
      if pos('.', PgmName) == 0 then leave
    end
    call close('Temp')
  end

  if ~exists(Storage'FWC'App'VersionInfo.txt') then address command 'version >'Storage'FWC'App'VersionInfo.txt 'PgmName

  call open('Temp', Storage'FWC'App'VersionInfo.txt')
    PgmVersion = readln('Temp')
  call close('Temp')

  if left(PgmVersion, 34) == 'Could not find version information' then do
    if App == 'FW' then do
      call open('Temp', CurrentDir''PgmName)
        /* Desired string at 325365 for v 5.06 */
        /* Desired string at 333771 for FW97   */
        FileOffset = 325300
        call seek('Temp', FileOffset, 'B')
        do until (EndPos ~= 0) | (PrevOffset = FileOffset)
          PrevOffset = FileOffset
          Chunk = readch('Temp', 10000)
          EndPos = pos('Created', Chunk)
          if EndPos == 0 then FileOffset = seek('Temp', -100, 'C')
        end
        if EndPos == 0 then PgmVersion = 'Final Writer - version unknown'
        else do
          StartPos = lastpos('Final', Chunk, EndPos)
          EndPos = pos('00'x||'00'x, Chunk, StartPos)
          PgmVersion = substr(Chunk, StartPos, EndPos - StartPos - 1)
        end
      call close('Temp')
      call open('Temp', Storage'FWC'App'VersionInfo.txt', 'W')
        call writeln('Temp', PgmVersion)
      call close('Temp')
    end
    else PgmVersion = PgmName" - can't find version info"
  end

  return
/**/

/***//*******  GetWidth (GW) Subroutine  ***********/
GetWidth:
  parse arg GW_ID

  if App = 'FW' then do
    GETOBJECTCOORDS GW_ID
    Parse Var result . . . GW_width .
  end
  else if App == 'PGS' then do
    SELECTOBJECT OBJECTID GW_ID  WINDOW winName
    GETOBJECT BOUNDINGBOX GW_Temp WINDOW winName
    GW_width = GW_Temp.Right - GW_Temp.Left
  end

  return GW_width
/**/

/***//*******  OpenBusy (OB) Subroutine  ***********/
OpenBusy:
  parse arg OB_BusyTitle, OB_EventCount

  OB_ProgressGroup=bguivgroup(,
        bguiinfo('OB_dummy',,'1B'x||'c'OB_BusyTitle)bguilayout(LGO_FixMinHeight,1)||,
        bguiprogress('OB_prog2_',,0,OB_EventCount)||,
        bguihgroup(,
                bguivarspace(50)bguilayout(LGO_FixMinHeight,1)||,
                bguibutton('OB_cancel_',Cancel$)bguilayout(LGO_FixMinHeight,1)||,
                bguivarspace(50)bguilayout(LGO_FixMinHeight,1),
        ,,,,'W'),
  ,-2,-2)

  OB_ProgressWindow = bguiwindow(PleaseWait$'...',OB_ProgressGroup,,2,,AppScreen)
  if bguiwinopen(OB_ProgressWindow) = 0 then call Cleanup

  Progress = 0

return OB_ProgressWindow
/**/

/***//*******  ParseVariables (PV) Subroutine  ***********/
ParseVariables:
parse arg PV_Line

PV_String = translate(PV_Line,,'=(+-*/,)"'||"'",' ')
PV_VarString = ''
PV_Var.      = '00'x
PV_LongVar   = 4
PV_LIT       = ''
PV_Count     = 0

do PV_i = 1 to words(PV_String)
  PV_Word = word(PV_String, PV_i)
  if pos(PV_Word'(', PV_Line) > 0 then iterate
  if datatype(PV_Word) == 'CHAR' then do
    if symbol(PV_Word) == 'LIT' then PV_LIT = PV_LIT''PV_Word', '
    if symbol(PV_Word) == 'VAR' then do
      PV_LongVar = max(PV_LongVar, length(PV_Word) + 2)
      if PV_Var.PV_Word == '00'x then do
        PV_Count = PV_Count + 1
        PV_Var.PV_Count = PV_Word
        PV_Var.PV_Word  = value(PV_Word)
      end
      if pos('.', PV_Word) > 0 then do
        PV_CompoundParts = subword(translate(PV_Word,,'.', ' '), 2)
        do PV_j = 1 to words(PV_CompoundParts)
          PV_Subword = word(PV_CompoundParts, PV_j)
          if PV_Var.PV_SubWord == '00'x then do
            PV_Count = PV_Count + 1
            PV_Var.PV_Count = PV_SubWord
            if symbol(PV_Subword) == 'LIT' then PV_Var.PV_SubWord  = 'LIT'
            else PV_Var.PV_SubWord  = value(PV_SubWord)
          end
        end
      end
    end
  end
end

do PV_i = 1 to PV_Count
  PV_Word = PV_Var.PV_i
  if length(PV_Var.PV_Word) > 50 then PV_Var.PV_Word = left(PV_Var.PV_Word, 50)'...'
  PV_Var.PV_Word = translate(PV_Var.PV_Word,,'0a'x||'0d'x||'00'x,'bb'x)
  PV_VarString = PV_VarString''right(PV_Word, PV_LongVar)' = 'PV_Var.PV_Word||'0a'x
end

if PV_LIT ~= '' then PV_VarString = right('LIT', PV_LongVar)' = 'strip(PV_LIT, 'B', ' ,')||'0a'x||PV_VarString
return PV_VarString
/**/

/***//*******  PathPart (PP) Subroutine ***********/
PathPart:
  parse arg PP_FileWithPath
  return left(PP_FileWithPath, max(lastpos(':', PP_FileWithPath), lastpos('/', PP_FileWithPath)))
/**/

/***//*******  PrintText (PT) Subroutine  ***********/
PrintText:
  parse arg PT_Left, PT_Top, PT_FontType, PT_Style, PT_Color, PT_Width, PT_Text

  if upper(PT_Style) == 'N' then PT_Font = Font.PT_FontType
  else PT_Font = Bold.PT_FontType

  if App == 'FW' then do
    if left(PT_Text, 1) == '"' then PT_Text = '""'PT_Text
    PT_Top = PT_Top + TextAdj * Height.PT_FontType
    TEXTBLOCKTYPEPREFS SIZE FSize.PT_FontType WIDTH trunc(PT_Width) COLOR '"'PT_Color'"' FONT PT_Font
    DRAWTEXTBLOCK 1 PT_Left PT_Top PT_Text; PT_id = result
  end
  else if App == 'PGS' then do
    DRAWTEXTOBJ PT_Left PT_Top WINDOW winName; PT_id = result
    SELECTTEXT AT PT_Left PT_Top WINDOW winName
    BEGINCOMMANDCAPTURE
      SETTYPESIZE FSize.PT_FontType WINDOW winName
      SETTYPEWIDTH PT_Width WINDOW winName
      SETFONT PT_Font WINDOW winName
      SETCOLORSTYLE '"'PT_Color'"' COLORNUMBER 0 FILL TEXT WINDOW winName
    ENDCOMMANDCAPTURE
    if pos('"', PT_Text) > 0 then do
      call open('IFile', Storage'Text2Insert.txt', 'W')
        call WriteLn('IFile', PT_Text)
      call close('IFile')
      INSERTTEXT FILE Storage'Text2Insert.txt' FILTER ASCII WINDOW winName
    end
    else INSERT '"'PT_Text'"' WINDOW winName
  end
  return PT_id
/**/

/***//*******  ProcessEvent (PE) Subroutine  ***********/
ProcessEvent:
  Day1 = ''
  Day2 = ''
  EnteredLine = 1
  Options = ''
  EnteredEvent = ''
  Box = 0
  Weekly = 0
  WindowRefreshed = 0

  if EventData == 0 then call CleanUp
  Parse Var EventData EventType ',' input
  EventType = strip(EventType)
  if EventType = Event$ then do
    Event.0   = 1
    Event.1   = input
    EventFile = ''
  end
  else do
    Parse var input RootDay ',' . ',' . ',' . ',' . ',' EventFile
    RootDay = strip(RootDay)
    EventFile = strip(EventFile)
    RootDay = ConvertDay(RootDay)

    call open('EventFile', EventFile)
      EventCount = 0
      do until eof('EventFile')
        Ln = ReadLn('EventFile')
        if eof('EventFile') == 0 then do
          if Ln == '' then iterate
          EventCount = EventCount + 1
          Event.EventCount = Ln
        end
      end
      Event.0 = EventCount
    call close('EventFile')
  end

  if Event.0 > 1 then Req = OpenBusy(ProcessEvents$'...', Event.0)
  if App == 'PGS' then do
    REFRESH OFF
  end
  do EC = 1 to Event.0
    if Req ~= 0 then call UpdateBusy(Req, 1)
    Box = 0
    Weekly = 0

    parse var Event.EC EnteredDay1 ',' EnteredDay2 ',' EnteredLine ',' Options ',' BoxInfo ',' EnteredEvent
    EnteredDay1  = strip(EnteredDay1)
    EnteredDay2  = strip(EnteredDay2)
    EnteredLine  = strip(EnteredLine)
    Options      = compress(upper(strip(Options)))
    BoxColor     = strip(word(BoxInfo, 1))
    BoxTint      = strip(word(BoxInfo, 2))
    if BoxTint == '' then BoxTint = 100
    EnteredEvent = strip(EnteredEvent)

    If EnteredDay2 == "" then EnteredDay2 = EnteredDay1
    If EnteredLine == '' then EnteredLine = 1
    if BoxColor == '' then BoxColor = Background.AddEvent

    if EventType = Event$ then do
      EnteredDay1 = ConvertDay(EnteredDay1)
      EnteredDay2 = ConvertDay(EnteredDay2)
    end
    else do
      EnteredDay1 = RootDay + EnteredDay1
      EnteredDay2 = RootDay + EnteredDay2
    end
    If EnteredDay1 > EnteredDay2 then Do
      TempDate = EnteredDay1
      EnteredDay1 = EnteredDay2
      EnteredDay2 = TempDate
    End

    if pos('B', Options) ~= 0 then Box = 1
    if pos('W', Options) ~= 0 then Weekly = 1

    /* Process Event */
    Text. = ''
    if App == 'PGS' then REFRESH OFF WINDOW winName
    do until Weekly = 0
      Event = EnteredEvent
      Line  = EnteredLine
      Day1  = EnteredDay1
      Day2  = EnteredDay2

      if Day1 > MaxDate then do
        Weekly = 0
        iterate
      end
      if Day2 > MaxDate then Day2 = MaxDate

      If Day1 ~= Day2 then Box = 1

      LineCount.DaySpan = 0
      Do until Day1 > Day2
        Day1Row = trunc((Day1 + StartDate - 1) / 7)
        Day2Row = trunc((Day2 + StartDate - 1) / 7)
        Day1Column = (Day1 + StartDate) - 7 * Day1Row - 1
        Day2Column = (Day2 + StartDate) - 7 * Day2Row - 1

        if Day1Row == Day2Row then DaySpan = Day2Column - Day1Column + 1
        else DaySpan = 7 - Day1Column
        AllowedWidth.DaySpan  = DaySpan * BoxWidth - Box * (2 * DateOffset)

        If Day1Row < 5 then BoxTop = CalTop + Day1Row * BoxHeight
        else BoxTop = CalTop + 4.5 * BoxHeight
        LeftEdge = Margin.Left + Day1Column * BoxWidth + DateOffset

        if (Text.0.DaySpan == '') & (event ~= '') then do
          Textline = 0
          Text.Textline.DaySpan = event

          /* Accomodate user line breaks */
          do until LineBreak = 0
            LineBreak = pos('//', Text.Textline.DaySpan)
            if LineBreak > 0 then do
              Nextline = Textline + 1
              Text.Nextline.DaySpan = substr(Text.Textline.DaySpan, LineBreak + 2)
              Text.Textline.DaySpan = left(Text.Textline.DaySpan, LineBreak - 1)
              Textline = Nextline
            end
          end
          Textline = 0

          /* Fit line(s) into allowable space */
          do until Text.Nextline.DaySpan == ''
            Nextline = Textline + 1
            if Box == 1 | Textline == 0 then Indent.Textline.DaySpan = 0
            else Indent.Textline.DaySpan = 3 * DateOffset

            if left(Text.Textline.DaySpan, length(TabSub)) == TabSub then do
              Indent.Textline.DaySpan = TabFactor * DateOffset
              Text.Textline.DaySpan = substr(Text.Textline.DaySpan, length(TabSub) + 1)
            end

            if App == 'FW' & length(Text.Textline.DaySpan) > 37 then do
              Wordbreak = lastpos(' ', Text.Textline.DaySpan, 37)
              Text.Nextline.DaySpan = strip(substr(Text.Textline.DaySpan, Wordbreak)' 'Text.Nextline.DaySpan)
              Text.Textline.DaySpan = strip(left(Text.Textline.DaySpan, Wordbreak))
            end
            ID = PrintText(1, 1, Highlight, 'N', Color.Highlight, Width.Highlight, Text.Textline.DaySpan)
            if App == 'FW' then redraw
            TextWidth.Textline.DaySpan = GetWidth(ID)
            if App == 'FW' then DELETEOBJECT ID
            else if App == 'PGS' then do
              SELECTOBJECT OBJECTID ID WINDOW winName
              DELETEOBJECT OBJECTID ID WINDOW winName
            end

            LineWidth.Textline.DaySpan = TextWidth.Textline.DaySpan + Indent.Textline.DaySpan
            if (LineWidth.Textline.DaySpan >= AllowedWidth.DaySpan) & (Words(Text.Textline.DaySpan) > 1) then do
              /* Move last word to next line */
              Wordbreak     = lastpos(' ', Text.Textline.DaySpan)
              Text.Nextline.DaySpan = strip(substr(Text.Textline.DaySpan, Wordbreak)' 'Text.Nextline.DaySpan)
              Text.Textline.DaySpan = strip(left(Text.Textline.DaySpan, Wordbreak))
            end
            else if Text.Nextline.DaySpan ~= '' then Textline = Textline + 1
          End
          LineCount.DaySpan = Textline
        end
        if Box then call DrawBox(LeftEdge, BoxTop + Line * Height.Highlight, AllowedWidth.DaySpan, Height.Highlight * (LineCount.DaySpan + 1), 'HL', Line.AddEvent, 1, BoxColor, BoxTint)
        if event ~= '' then
          do i = 0 to LineCount.DaySpan
            Text.Top = BoxTop + (Line + i) * Height.Highlight
            if Box == 0 then Text.Left = LeftEdge + Indent.i.DaySpan
            else Text.Left = LeftEdge + (AllowedWidth.DaySpan - TextWidth.i.DaySpan) / 2
            TextWidth = min(Width.Highlight, Width.Highlight * AllowedWidth.DaySpan / TextWidth.i.DaySpan - 4)
            call PrintText(Text.Left, Text.Top, Highlight, 'N', Color.AddEvent, TextWidth, Text.i.DaySpan)
          end

        Day1 = Day1 + DaySpan
        if Day1 > Day2 then leave
        else if trunc((Day1 + StartDate - 1) / 7) > 4 & Day2 > MonthLength.Month then Day2 = Day1
      end

      if Weekly == 1 then do
        EnteredDay1 = EnteredDay1 + 7
        EnteredDay2 = EnteredDay2 + 7
      end
    end

    if App == 'FW' then redraw
    else if App == 'PGS' then SELECTOBJECT None WINDOW winName
  end

  if Req ~= 0 then call bguiwinclose(Req)

  if App == 'PGS' then do
    REFRESH ON WINDOW winName
    REFRESHWINDOW WINDOW winName
    WindowRefreshed = 1
  end

return
/**/

/***//*******  Syntax () Subroutine  ***********/
Syntax:
  signal off syntax

  ErrorLine  = SIGL
  SourceLine = strip(SourceLine(ErrorLine))
  if SourceLine = 'bguiopen = bguiopen()' then do
    bguiopen = 0
    call AddMsg('E', 'Could not open bgui functions. Check versions:')
    call AddMsg('E', '  bgui.library v41+')
    call AddMsg('E', '  rexxbgui.library v4+')
    call Cleanup
  end

  call AddMsg('E', 'Error 'RC' ('errortext(RC)')')
  call AddMsg('E', 'Line 'ErrorLine': 'SourceLine)
  call AddMsg('E', ParseVariables(SourceLine))

  call Cleanup
  exit
/**/

/***//*******  UpdateBusy (UB) Subroutine  ***********/
UpdateBusy:
  parse arg UB_ReqWin, UB_ProgressMade

  if Req ~= 0 then do
    Progress = Progress + UB_ProgressMade

    call bguiset(obj.OB_prog2_,UB_ReqWin,PROGRESS_Done,Progress)
    if bguiwinevent(UB_ReqWin,'ID') == id.OB_cancel_ then call Cleanup
  end

  return
/**/

/***//*******  SetVariables Subroutine  ***********/
SetVariables:

/***//* Initialize Variables */
  AddEventRows    = 9
  DoDebug         = 0
  DoShanghai      = 1
  esc             = "1B"x
  EventFile       = ''
  EventKey        = 'E'
  FSize.          = 10
  Highlight       = 0
  PatVar          = '#?.data'
  Req             = 0
  StartWeek       = 0
  Storage         = 'RAM:FWC/'
  TabFactor       = 3
  TabSub          = '/~'
  Width.          = 100

  if App == 'FW' then Font. = "FWFonts/SWOLFonts/SoftSans"
  else if App == 'PGS' then Font. = 'Helvetica-Normal'

  TextAdj         = 0.77
  WTextArea       = 0.20  /* fraction of print height used for top of calendar (Wide) */
  TTextArea       = 0.15  /* fraction of print height used for top of calendar (Tall) */
  DateOffset      = 0.02  /* fraction of box width to offset dates from edge of box   */

  Sunday$    = 'Sunday'
  Monday$    = 'Monday'
  Tuesday$   = 'Tuesday'
  Wednesday$ = 'Wednesday'
  Thursday$  = 'Thursday'
  Friday$    = 'Friday'
  Saturday$  = 'Saturday'

  D.0 = 'Sunday'
  D.1 = 'Monday'
  D.2 = 'Tuesday'
  D.3 = 'Wednesday'
  D.4 = 'Thursday'
  D.5 = 'Friday'
  D.6 = 'Saturday'

  January$   = 'January'
  February$  = 'February'
  March$     = 'March'
  April$     = 'April'
  May$       = 'May'
  June$      = 'June'
  July$      = 'July'
  August$    = 'August'
  September$ = 'September'
  October$   = 'October'
  November$  = 'November'
  December$  = 'December'

  MonthLength.1    = 31
  MonthLength.2    = 28
  MonthLength.3    = 31
  MonthLength.4    = 30
  MonthLength.5    = 31
  MonthLength.6    = 30
  MonthLength.7    = 31
  MonthLength.8    = 31
  MonthLength.9    = 30
  MonthLength.10   = 31
  MonthLength.11   = 30
  MonthLength.12   = 31

  Black$          = 'Black'
  BoxColor$       = 'Box Colo_r:'
  Boxed$          = '_Boxed:'
  Cancel$         = '_Cancel'
  CantFind$       = "can't be found"
  End$            = 'End:'
  EnterEvent$     = 'You must enter an event...'
  EnterEventInfo$ = 'Enter event information:'
  EnterStartdate$ = 'You must enter a start date...'
  Event$          = 'Event:'
  File$           = 'File:'
  Line$           = '_Line:'
  Notice$         = 'notice'
  OK$             = '_OK'
  PleaseWait$     = 'Please wait'
  PrepReq$        = 'Preparing requester'
  ProcessEvents$  = 'Processing events'
  SelectFile$     = 'Select data file:'
  Start$          = 'Start:'
  Weekly$         = '_Weekly:'
  White$          = 'White'
/**/

  ProcessNow = 'WhiteName BlackName DoShanghai EventKey Storage'

  if exists(ScriptDir'FWCalendar.data') then do
    call open('DataFile', ScriptDir'FWCalendar.data')
      do until eof('DataFile')
        Ln = ReadLn('DataFile')
        if pos(upper(word(Ln, 1)), upper(ProcessNow)) ~= 0 then interpret Ln
        else if right(word(Ln, 1), 1) == '$' then interpret Ln
        else if pos('End Pass One', Ln) > 0 then leave
      end
    call close('DataFile')
  end

  call makedir(left(Storage, length(Storage) - 1))

  if Symbol('BlackName') == 'VAR' then Black$ = BlackName
  if Symbol('WhiteName') == 'VAR' then White$ = WhiteName

  Color.          = Black$
  Line.           = Black$
  Background.     = White$
  Color.White     = White$
  Type.0          = Event$
  Type.1          = File$

  AppScreen = ''
  DefScreen = ''
  if (RexxTricks == 1) & (DoShanghai ~= 0) then do
    if (pubscreenlist('ScreenList') > 0) then do
      do i = 1 to ScreenList.0
        if pos(AppName, upper(ScreenList.i)) > 0 then do
          AppScreen = ScreenList.i
          DefScreen = setdefaultpubscreen(AppScreen)
          leave
        end
      end
    end
  end

  /**** Read user variables ****/
  if exists(ScriptDir'FWCalendar.data') then do
    call open(DataFile, ScriptDir'FWCalendar.data')
      do until eof(DataFile)
        Ln = ReadLn(DataFile)
        if left(Ln, 15) == '/* End Pass One' then do
          HighlightStart = seek(DataFile, 0)
          leave
        end
        interpret Ln
      end
    call close(DataFile)
  end

  FSize.4pt       = 4

  do i = 0 to 6
    val = i - StartWeek
    if val < 0 then val = 7 + val
    interpret 'Day.'D.i '=' val
    interpret 'Day.val = 'D.i'$'
  end

  Month.1  = January$
  Month.2  = February$
  Month.3  = March$
  Month.4  = April$
  Month.5  = May$
  Month.6  = June$
  Month.7  = July$
  Month.8  = August$
  Month.9  = September$
  Month.10 = October$
  Month.11 = November$
  Month.12 = December$

  do i = 1 to 12
    AbbrMonth.i  = left(Month.i, 3)
  end

  if App == 'FW' then do
    GETPAGESETUP ORIENT; FWC_Orientation = result
    if FWC_Orientation == 'Wide' then TextArea = WTextArea
    else TextArea = TTextArea

    GETDISPLAYPREFS Measure; UserPrefs = 'DISPLAYPREFS Measure 'result
    DISPLAYPREFS Measure Inches
    GETSECTIONSETUP Top Bottom Inside Outside
    parse var result Margin.Top Margin.Bottom Margin.Left Margin.Right

    GETPAGESETUP Width Height
    parse var result FullWidth FullHeight

    GETTEXTBLOCKTEXT 2; TempDate = result
    if (left(TempDate, 3) ~= 'FWC') | (length(TempDate) ~= 11) then do
      call AddMsg('E', 'FWCAddEvent.rexx can only be run on a MONTHLY calendar')
      call AddMsg('E', '  created by FWCalendar.rexx and currently loaded.')
      call Cleanup
    end
    else TempDate = substr(TempDate, 4)

    TextBlockPrefs TEXTFLOW None
  end
  else if App = 'PGS' then do
    CURRENTWINDOW; winName = '"'RESULT'"'
    GETMASTERPAGES MPage; PageName = MPage.0
    GETMEASUREMENTS COORDINATE stemc RELATIVE rel TEXT tex FROM fro
    UserPrefs = 'SETMEASUREMENTS COORDINATE 'stemc.horizontal stemc.vertical' RELATIVE 'rel' TEXT 'tex' FROM 'fro
    SETMEASUREMENTS COORDINATE Inches Sameas RELATIVE Sameas TEXT Points FROM Page
    GETMARGINGUIDES temp
    Margin.Left   = temp.inside
    Margin.Right  = temp.outside
    Margin.Top    = temp.top
    Margin.Bottom = temp.bottom

    GETDIMENSIONS layout MASTERPAGE "'"PageName"'"
    if layout.orientation == 'LANDSCAPE' then do
      TextArea   = WTextArea
      FullWidth  = layout.height
      FullHeight = layout.width
    end
    else do
      TextArea   = TTextArea
      FullWidth  = layout.width
      FullHeight = layout.height
    end

    SELECTTEXT at 0 0 WINDOW winName
    SELECTTEXT ALL WINDOW winName
    EXPORTTEXT AMIGA FILE Storage"TempDate.txt" FILTER "ASCII" STATUS FORCE
    if exists(Storage"TempDate.txt") then do
      open(TDFile, Storage"TempDate.txt")
        TempDate = ReadLn(TDFile)
      close(TDFile)
    end
    if (left(TempDate, 3) ~= 'FWC') | (length(TempDate) ~= 11) then do
      call AddMsg('E', 'FWCAddEvent.rexx can only be run on a MONTHLY calendar')
      call AddMsg('E', '  created by FWCalendar.rexx and currently loaded.')
      call Cleanup
    end
    else TempDate = substr(TempDate, 4)
  end

  PrintWidth       = FullWidth - Margin.Left - Margin.Right
  PrintHeight      = FullHeight - Margin.Top - Margin.Bottom

  if App == 'FW' then do
    GETOBJECTCOORDS 2; Parse Var result . . . . Height.4pt
  end
  else if App == 'PGS' then Height.4pt = GetHeight(4pt)
  if ((PrintHeight - Height.4pt - (TextArea * PrintHeight))/5 * 8) >= 4 then
      PrintHeight = PrintHeight - Height.4pt

  BoxWidth         = PrintWidth/7
  CalRight         = Margin.Left + BoxWidth * 7
  TextArea         = TextArea * PrintHeight
  CalTop           = TextArea + Margin.Top
  BoxHeight        = (PrintHeight - TextArea)/5
  DateOffset       = DateOffset * BoxWidth
  FSize.Highlight  = BoxHeight/AddEventRows * 72
  if App == 'FW' then FSize.Highlight = max(trunc(FSize.Highlight), 4)
  Height.Highlight = GetHeight(Highlight)
  RowsThatFit      = trunc(BoxHeight / Height.Highlight + 0.05)

  VariablesSet = 1
return
/**/

